"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Schema = void 0;
const ValidationResults_1 = require("../models/ValidationResults");
class Schema {
    constructor(schema, rules) {
        this.schema = schema;
        this.rules = rules || {};
    }
    check(input) {
        var _a;
        try {
            const inputKeysLength = Object.keys(input).length;
            const expectedKeysLength = Object.keys(this.schema).length;
            if (inputKeysLength < expectedKeysLength) {
                return (0, ValidationResults_1.MISSING_KEYS_INPUT)();
            }
            else if (inputKeysLength > expectedKeysLength && !((_a = this.rules.excess) === null || _a === void 0 ? void 0 : _a.allow)) {
                return (0, ValidationResults_1.EXCESS_KEYS_INPUT)();
            }
            for (const [key, type] of Object.entries(this.schema)) {
                if (input[key] === undefined) {
                    return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
                }
                const validation = this.isOfType(input[key], type);
                if (!(validation === null || validation === void 0 ? void 0 : validation.success)) {
                    return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
                }
            }
            return (0, ValidationResults_1.VALID_INPUT)(input);
        }
        catch (error) {
            return (0, ValidationResults_1.INTERNAL_ERROR)();
        }
    }
    isOfType(
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    value, expectedType) {
        try {
            if (value === undefined) {
                return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
            }
            if (expectedType === 'any') {
                return (0, ValidationResults_1.VALID_INPUT)(value);
            }
            else if (expectedType === 'array') {
                if (!Array.isArray(value)) {
                    return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
                }
            }
            else if (typeof value !== expectedType) {
                return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
            }
            return (0, ValidationResults_1.VALID_INPUT)(value);
        }
        catch (error) {
            console.log(error);
            return (0, ValidationResults_1.INTERNAL_ERROR)();
        }
    }
    length() {
        return Object.keys(this.schema).length;
    }
}
exports.Schema = Schema;
